/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

namespace Diorite.Widgets{
	/**
	 * Simple Error dialog with OK button.
	 */
	public class ErrorDialog: Gtk.MessageDialog{
		
		/**
		 * Constructs new error dialog.
		 * 
		 * @param title Title of the error message (not title of the dialog)
		 * @param message Text of the error message
		 */
		public ErrorDialog(string title, string message){
			Object(
				title: "",
				modal: true,
				message_type: Gtk.MessageType.ERROR,
				buttons: Gtk.ButtonsType.OK
			);
			this.text = title;
			this.secondary_text = message;
		}
		
		public override void response(int id){
			this.destroy();
		}
	}
	
	/**
	 * Simple confirmation dialog with YES and NO buttons.
	 */
	public class ConfirmDialog: Gtk.MessageDialog{
		
		/**
		 * Constructs new error dialog.
		 * 
		 * @param title Title of the error message (not title of the dialog)
		 * @param message Text of the error message
		 */
		public ConfirmDialog(string title, string message){
			Object(
				title: "",
				modal: true,
				message_type: Gtk.MessageType.QUESTION,
				buttons: Gtk.ButtonsType.YES_NO
			);
			this.text = title;
			this.secondary_text = message;
		}
		
		public override void response(int id){
			this.destroy();
		}
	}
	
	/**
	 * Simple Information dialog with OK button.
	 */
	public class InfoDialog: Gtk.MessageDialog{
		
		/**
		 * Constructs new error dialog.
		 * 
		 * @param title Title of the error message (not title of the dialog)
		 * @param message Text of the error message
		 */
		public InfoDialog(string title, string message){
			Object(
				title: "",
				modal: true,
				message_type: Gtk.MessageType.INFO,
				buttons: Gtk.ButtonsType.OK
			);
			this.text = title;
			this.secondary_text = message;
		}
		
		public override void response(int id){
			this.destroy();
		}
	}
	
	 /**
     * Button with stock icon and custom label
     */
    public class ImageLinkButton: Gtk.LinkButton{
		/**
		 * Create new button with stock icon and custom label
		 * @param image Stock id for icon
		 * @param uri target URI
		 * @param label text on the button
		 */
		public ImageLinkButton(string image, string uri, string label){
			Object(label: label, uri: uri);
			xalign = 0.5f;
			yalign = 0.5f;
			var box = new Gtk.VBox(false, 5);
			var child = this.get_child();
			this.remove(child);
			var img = new Gtk.Image.from_file(image);
			box.pack_start(img, false, false);
			box.pack_start(child, true, true);
			this.add(box);
			box.show_all();
		}
	}
	
}
