/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if UNITY_QUICK_LIST

using Diorite.Logger;
using Diorite.System;
using Unity;

namespace Nuvola.Extensions.UnityQuickList{

public Nuvola.ExtensionInfo get_info(){
	return {
		/// Name of a plugin providing scrobbling to Last.fm
		_("Unity Quick List"),
		Nuvola.VERSION,
		/// Extension descriptiom
		_("This plugin provides integration with docks compliant with Unity Quick List (Unity Launcher, DockBarX)."),
		"Jiří Janoušek",
		typeof(Extension)
	};
}

/**
 * Manages dock item at Unity Launcher
 */
public class Extension: Nuvola.Extension{
	private weak Diorite.Application app;
	private weak Diorite.Actions actions;
	private weak Nuvola.Player player;
	private LauncherEntry dock_item;
	private bool active = true;
	
	/**
	 * {@inheritDoc}
	 */
	public override void load(ObjectContainer objects) throws ExtensionError{
		// Get dependencies, throws ExtensionError
		app = objects.get<Diorite.Application>("application");
		actions = objects.get<Diorite.Actions>("actions");
		player = objects.get<Player>("player");
		
		debug("Unity Launcher Entry for %s.desktop", app.desktop_entry);
		dock_item = LauncherEntry.get_for_desktop_id(
			app.desktop_entry + ".desktop");
		var menu = new Dbusmenu.Menuitem();
		foreach(var name in PLAYER_ACTIONS_ALL){
			var action = actions.get_action(name);
			if(action != null){
				menu.child_append(create_menu_item(action));
			}
		}
		dock_item.quicklist = menu;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public override void unload(){
		remove_menu();
	}
	
	private Dbusmenu.Menuitem create_menu_item(Diorite.Action action){
		var item = new Dbusmenu.Menuitem();
		item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, action.label);
		item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, action.stock_id);
		item.property_set_bool(Dbusmenu.MENUITEM_PROP_VISIBLE, action.sensitive);
		item.item_activated.connect(() => {action.activate();});
		action.notify.connect((o, p) => {
			switch(p.name){
			case "sensitive":
				item.property_set_bool(Dbusmenu.MENUITEM_PROP_VISIBLE, action.sensitive);
				break;
			case "label":
				item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, action.label);
				break;
			case "stock-id":
				item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, action.stock_id);
				break;
			}
		});
		return item;
	}
			
	private void remove_menu(){
		lock(this.active){
			if(!this.active) return;
			this.active = false;
			if(this.dock_item != null && this.dock_item.quicklist != null){
				this.dock_item.quicklist = null;
			}
			
		}
	}
}

} // namespace Nuvola.Extensions.UnityQuickList
#endif
