/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

public class Nuvola.CrashDialog: Gtk.Dialog{
	public Gtk.CheckButton disable_check;
	private Diorite.ActionCheckbox[] actions;
	
	#if WEB_CACHE
		public Gtk.CheckButton purge_browser;
	#endif
	
	public CrashDialog(Diorite.Actions actions, bool soup_buggy){
		/// Title of the Crash dialog
		GLib.Object(title: _("Nuvola Player Crash Dialog"));
		
		var bug_button = new Gtk.Button.with_label(_("Report a problem"));
		var action_area = get_action_area() as Gtk.ButtonBox;
		action_area.pack_start(bug_button, false, false);
		bug_button.clicked.connect(() => {
			Diorite.System.open_uri_with_fallback(BUG_TRACKER);
		});
		this.add_button(Gtk.Stock.OK, Gtk.ResponseType.OK);
		action_area.layout_style = Gtk.ButtonBoxStyle.EDGE;
		
		var content = this.get_content_area() as Gtk.Box;
		content.border_width = 35;
		content.spacing = 0;
		Gtk.Label label;
		const int xpad = 10;
		
		var box = new Gtk.VBox(false, 0);
		box.border_width = 15;
		content.pack_start(box, false, false, 0);
		
		/// Crash dialog
		label = new Gtk.Label(_("<b>Previous Nuvola Player session was not closed correctly.</b>"));
		label.xpad = xpad;
		label.use_markup = true;
		label.wrap_mode = Pango.WrapMode.WORD;
		label.wrap = true;
		label.justify = Gtk.Justification.FILL;
		box.pack_start(label, false, false, 0);
		
		/// Crash dialog
		label = new Gtk.Label(_("You can try following options to solve this issue:"));
		label.xalign = 0;
		label.wrap_mode = Pango.WrapMode.WORD;
		label.wrap = true;
		label.justify = Gtk.Justification.FILL;
		box.pack_start(label, false, false, 10);
		
		string[] ACTIONS = {Actions.PURGE_CACHE, Actions.PURGE_UI_STATE, Actions.PURGE_COOKIES, Actions.PURGE_SETTINGS, Actions.DISABLE_PROXY};
		this.actions = {};
		foreach(var action_id in ACTIONS){
			var action = actions.get_action(action_id);
			if(action != null){
				var checkbox = new Diorite.ActionCheckbox(action);
				this.actions += checkbox;
				box.pack_start(checkbox, false, false, 0);
			}
		}
		
		label = new Gtk.Label(
			/// Crash dialog
			_("Cloud music services use Flash to play music. Unfortunately, Flash Player is known not to be very stable and may cause crash of Nuvola Player. Please visit <a href='%s'>Flash troubleshooting</a>")
			.printf("https://blueprints.launchpad.net/nuvola-player/+spec/flash-plugin-issues"));
		label.xalign = 0;
		label.xpad = 15;
		label.ypad = 15;
		label.use_markup = true;
		label.wrap_mode = Pango.WrapMode.WORD;
		label.wrap = true;
		label.justify = Gtk.Justification.FILL;
		
		content.pack_start(label, true, true, 0);
		
		label = new Gtk.Label(
			/// Crash dialog
			_("Another cause may be session/login manager, if it doesn't notify Nuvola Player to quit properly when user is about to logout.")
			);
		label.xalign = 0;
		label.xpad = 15;
		label.ypad = 0;
		label.use_markup = true;
		label.wrap_mode = Pango.WrapMode.WORD;
		label.wrap = true;
		label.justify = Gtk.Justification.FILL;
		
		content.pack_start(label, true, true, 0);
		
		box = new Gtk.VBox(false, 0);
		box.border_width = 15;
		content.pack_start(box, false, false, 0);
		
		/// Crash dialog
		this.disable_check = new Gtk.CheckButton.with_label(_("Do not show crash dialog again."));
		box.pack_start(this.disable_check, false, false, 0);
		content.show_all();
		
	}
	
	public void run_actions(){
		foreach(var action in actions){
			action.run();
		}
	}
}
