/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

namespace Nuvola{

public class ServicesManagerView: Gtk.Table{
	private static const int ICON_SIZE = 26; 
	private Gtk.TreeView selector;
	private ServicesManager manager;
	private HashTable<string, Service> services;
	private StaticWebView page;
	public Gtk.ButtonBox buttons{ get; private set;}
	private File template;
	private string default_icon_name;
	private Gtk.Button button_remove;
	
	private Service? _service = null;
	public Service? service{
		get{
			return _service;
		}
		private set{
			_service = value;
			if(value != null){
				button_remove.sensitive = service.removable;
				load_service_page(value);
			}
			else{
				button_remove.sensitive = false;
			}
		}
	}
	
	public signal void response(int response_id);
	
	public ServicesManagerView(string default_icon_name, File template, ServicesManager manager, string? selected_id=null){
		this.manager = manager;
		this.services = manager.list_services();
		this.template = template;
		this.default_icon_name = default_icon_name;
		
		
		// id, pixbuf, name
		var model = new Gtk.ListStore(SMI.COUNT, typeof(string), typeof(Gdk.Pixbuf), typeof(string));
		
		var opts = Gtk.AttachOptions.SHRINK;
		
		var win = new Gtk.ScrolledWindow(null, null);
		selector = new Gtk.TreeView();
		var column = new Gtk.TreeViewColumn();
		var icon_renderer = new Gtk.CellRendererPixbuf();
		icon_renderer.xpad = 5;
		var name_renderer = new Gtk.CellRendererText();
		name_renderer.ypad = 10;
		/// Header in services selector
		column.set_title(_("Streaming services"));
		column.pack_start(icon_renderer, false);
		column.set_attributes(icon_renderer, "pixbuf", SMI.ICON); 
		column.pack_start(name_renderer, true);
		column.set_attributes(name_renderer, "markup", SMI.NAME);
		column.sort_column_id = SMI.NAME;
		selector.append_column(column);
		
		win.add(selector);
		win.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
		win.shadow_type = Gtk.ShadowType.IN;
		border_width = 3;
		win.border_width = 3;
		attach(win, 0, 1, 0, 1, Gtk.AttachOptions.FILL, Gtk.AttachOptions.FILL|Gtk.AttachOptions.EXPAND, 0, 0);
		
		var manager_buttons = new Gtk.HButtonBox();
		manager_buttons.layout_style = Gtk.ButtonBoxStyle.SPREAD;
		attach(manager_buttons, 0, 1, 1, 2, Gtk.AttachOptions.FILL, opts, 0, 0);
		
		var button_add = new Gtk.Button.from_stock(Gtk.Stock.ADD);
		button_add.clicked.connect(on_add_service_clicked);
		manager_buttons.add(button_add);
		
		button_remove = new Gtk.Button.from_stock(Gtk.Stock.REMOVE);
		button_remove.sensitive = false;
		button_remove.clicked.connect(on_remove_service_clicked);
		manager_buttons.add(button_remove);
		
		page = new StaticWebView();
		var page_scroll = new Gtk.ScrolledWindow(null, null);
		page_scroll.shadow_type = Gtk.ShadowType.IN;
		page_scroll.add(page);
		attach(page_scroll, 1, 2, 0, 1,
			Gtk.AttachOptions.EXPAND|Gtk.AttachOptions.FILL,
			Gtk.AttachOptions.EXPAND|Gtk.AttachOptions.FILL, 3, 3);
		
		buttons = new Gtk.HButtonBox();
		buttons.layout_style = Gtk.ButtonBoxStyle.SPREAD;
		attach(buttons, 1, 2, 1, 2, Gtk.AttachOptions.FILL|Gtk.AttachOptions.EXPAND, Gtk.AttachOptions.SHRINK, 0, 0);
		
		var selection = selector.get_selection();
		selection.changed.connect(on_selection_changed);
		selection.set_mode(Gtk.SelectionMode.SINGLE);
		selector.row_activated.connect(on_row_activated);
		
		load(model, selected_id);
	}
	
	#if DEBUG_MEMORY
	~ServicesManagerView(){
		debug("~ServicesManagerView()");
	}
	#endif
	
	private void load(Gtk.ListStore model, string? selected_id=null){
		string? selected = null;
		if(selected_id != null){
			selected = selected_id;
		}
		else if(this.service != null){
			selected = this.service.id;
		}
		
		var file = template.get_parent().get_child("_resources").get_child("nuvolaplayer.png");
		Gdk.Pixbuf? default_icon = null;
		try{
			default_icon = Gtk.IconTheme.get_default().load_icon(default_icon_name, ICON_SIZE, 0);
		}
		catch(Error e){
			warning("Unable to load application icon.");
		}
		
		var keys = services.get_keys();
		keys.sort(strcmp);
		Gtk.TreeIter iter;
		Gtk.TreeIter? current_service = null;
		foreach(var key in keys){
			var service = services.lookup(key);
			return_if_fail(service != null);
			model.append(out iter);
			file = service.resources.get_child("icon.svg");
			Gdk.Pixbuf pixbuf;
			try{
				pixbuf = new Gdk.Pixbuf.from_file_at_size(file.get_path(), ICON_SIZE, ICON_SIZE);
			}
			catch(GLib.Error e){
				try{
					file = service.resources.get_child("icon.png");
					pixbuf = new Gdk.Pixbuf.from_file_at_size(file.get_path(), ICON_SIZE, ICON_SIZE);
				}
				catch(GLib.Error e){
					pixbuf = default_icon;
				}
			}
			model.set(iter, SMI.ID, key, SMI.ICON, pixbuf, SMI.NAME,
				"<span size=\"16000\"> " + Markup.escape_text(service.name) + "</span>");
			if(service.id == selected){
				current_service = iter;
			}
		}
		
		selector.model = model;
		
		var selection = selector.get_selection();
		if(current_service != null || model.get_iter_first(out current_service)){
			selection.select_iter(current_service);
		}
		selector.grab_focus();
	}
	
	public void reload(string? selected_id=null){
		var model = selector.model as Gtk.ListStore;
		selector.model = null;
		model.clear();
		services = manager.list_services();
		load(model, selected_id);
	}
	
	public void on_row_activated(Gtk.TreePath path, Gtk.TreeViewColumn column){
		var model = selector.model as Gtk.ListStore;
		Gtk.TreeIter iter;
		string id;
		if(model.get_iter(out iter, path)){
			model.get(iter, SMI.ID, out id);
			service = services.lookup(id);
			/* Idle callback prevents gdk_window_get_cursor: assertion `GDK_IS_WINDOW (window)' failed */
			Idle.add(response_ok_cb);
		}
	}
	
	private bool response_ok_cb(){
		response(Gtk.ResponseType.OK);
		return false;
	}
	
	public void add_button(Gtk.Button button, int response_id = Gtk.ResponseType.NONE){
		if(response_id != Gtk.ResponseType.NONE){
			button.clicked.connect(() => {
				response(response_id);
			});
		}
		buttons.add(button);
	}
	
	private void on_selection_changed(Gtk.TreeSelection selection){
		var model = selector.model as Gtk.ListStore;
		Gtk.TreeIter iter;
		string id;
		if(selection.get_selected(null, out iter)){
			model.get(iter, SMI.ID, out id);
			service = services.lookup(id);
		}
	}
	
	private void load_service_page(Service service){
		var builder = new StringBuilder();
		var desc = service.description;
		var version = "%d.%d".printf(service.version, service.version_minor);
		var maintainer_name = Markup.escape_text(service.maintainer_name);
		var maintainer_link = Markup.escape_text(service.maintainer_link);
		var resources = service.resources.get_uri() + "/";
		string requirements;
		int faq;
		if(service.flash_plugin){
			/// Warning in service selector if service requires Flash
			requirements = _("This streaming service requires Adobe Flash plugin to work properly.");
			if(!manager.compatible_flash_plugin){
				requirements = "%s<br /><span style=\"color: red;\">%s</span>".printf(
					/// Warning in service selector if compatible Flash is not found
					requirements, _("No compatible Flash plugin has been found."));
			}
			faq = 2107;
		}
		else{
			/// Message in service selector if service doesn't require Flash
			requirements = _("This streaming service may require MP3 decoder to work properly.");
			faq = 2011;
		}
		requirements = "%s <a href=\"%s%d\">%s</a>".printf(requirements,
			"https://answers.launchpad.net/nuvola-player/+faq/", faq,
			/// Text of a link to FAQ about Audio Playback Requirements
			_("More details."));
		
		try{
			var dis = new DataInputStream(template.read());
			string line;
			while ((line = dis.read_line (null)) != null){
				line = line.replace("${service.description}", desc);
				line = line.replace("${service.name}", service.name);
				line = line.replace("${service.version}", version);
				line = line.replace("${service.maintainer_name}", maintainer_name);
				line = line.replace("${service.maintainer_link}", maintainer_link);
				line = line.replace("${service.resources}", resources);
				line = line.replace("${service.screenshots}", SCREENSHOTS_URL);
				line = line.replace("${service.requirements}", requirements);
				builder.append(line);
				builder.append_c('\n');
			}
			dis.close();
		}
		catch(Error e){
			debug("ConfigFile: Unable to read %s", template.get_uri());
		}
		page.static_uri = template.get_uri();
		page.load_string((owned) builder.str, "text/html", "utf-8", template.get_uri());
	}
	
	private void on_add_service_clicked(){
		var dialog = new Gtk.FileChooserDialog(_("Open service integration package"),
			get_toplevel() as Gtk.Window, Gtk.FileChooserAction.OPEN, Gtk.Stock.CANCEL,
			Gtk.ResponseType.CANCEL, Gtk.Stock.OPEN, Gtk.ResponseType.ACCEPT);
		dialog.set_default_size(400, -1);
		var box = dialog.get_content_area() as Gtk.Box;
		if(box != null){
			var label = new MultilineLabel.with_markup(
			"New or updated service integrations are available at <a href='%s'>Nuvola Player Project website</a>.".printf(Nuvola.UPDATE_PACKS_URL)
			+ " " + _("You can follow Nuvola Player on <a href='%1$s'>Google+</a>, <a href='%2$s'>Facebook</a> or <a href='%3$s'>Twitter</a> to be informed about new or updated packages.").printf(GOOGLE_PLUS_URL, FACEBOOK_URL, TWITTER_URL));
			box.pack_start(label, false, false, 5);
			box.reorder_child(label, 0);
			label.show();
		}
		var response = dialog.run();
		var file = dialog.get_file();
		dialog.destroy();
		
		if(response == Gtk.ResponseType.ACCEPT){
			try{
				var service = manager.install_service(file);
				var info = new Diorite.Widgets.InfoDialog(_("Installation successfull"),
					_("Service %1$s (version %2$d.%3$d) has been installed succesfuly").printf(service.name, service.version, service.version_minor));
				
				reload(service.id);
				info.run();
			}
			catch(ServiceError e){
				var error = new Diorite.Widgets.ErrorDialog(_("Installation failed"),
					_("Installation of service from package %s failed.").printf(file.get_path())
					+ "\n\n" + e.message);
				error.run();
			}
		}
	}
	
	private void on_remove_service_clicked(){
		try{
			manager.remove_service(service);
			var info = new Diorite.Widgets.InfoDialog(_("Removal successfull"),
				_("Service %1$s (version %2$d.%3$d) has been succesfuly removed").printf(service.name, service.version, service.version_minor));
			service = null;
			reload();
			info.run();
		}
		catch(ServiceError e){
			var error = new Diorite.Widgets.ErrorDialog(_("Removal failed"),
				_("Removal of service %s failed.").printf(service.name)
				+ "\n\n" + e.message);
			service = null;
			reload();
			error.run();
		}
	}
	
	private enum SMI{
		ID,
		ICON,
		NAME,
		COUNT;
	}
}




public class BoldLabel: Gtk.Label{
	public BoldLabel(string? label = null){
		Object(label: label);
		var font = new Pango.FontDescription();
		font.set_weight(Pango.Weight.HEAVY);
		//font.merge(font2, true);
		modify_font(font);
	}
}

public class MultilineLabel: Gtk.Label{
	public MultilineLabel(string text){
		Object(label: text, xalign: 0, wrap_mode: Pango.WrapMode.WORD, justify: Gtk.Justification.FILL);
		set_line_wrap(true);
		set_size_request(0, -1);
		width_chars = 10;
		max_width_chars = 10;
	}
	
	public MultilineLabel.with_markup(string text){
		this(text);
		use_markup = true;
	}
}

} // namespace Nuvola
