/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * WebView which has static content and all links are opened in
 * a web browser.
 */
public class Nuvola.StaticWebView: WebKit.WebView{
	public string? static_uri{ get; set; default = null;}
	
	/**
	 * Constructs new Static Web View
	 * 
	 * @param body HTML body to render
	 */
	public class StaticWebView(File? file=null, bool plugins=false){
		var settings =  get_settings();
		settings.enable_caret_browsing = false;
		settings.enable_plugins = plugins;
		settings.javascript_can_open_windows_automatically = false;
		settings.enable_site_specific_quirks  = false;
		settings.enable_default_context_menu = Args.debug_javascript;
		settings.enable_developer_extras = Args.debug_javascript;
		set_settings(settings);
		full_content_zoom = true;
		if(file != null){
			static_uri = file.get_uri();
			load_uri(file.get_uri());
		}
		navigation_policy_decision_requested.connect(on_page_change);
		new_window_policy_decision_requested.connect(on_page_change);
		
		if(Args.debug_javascript){
			new InspectorWindow(this);
		}
	}
	
	#if DEBUG_MEMORY
	~StaticWebView(){
		debug("~StaticWebView");
	}
	#endif
	
	private bool on_page_change(WebKit.WebFrame frame, WebKit.NetworkRequest req,
		WebKit.WebNavigationAction action, WebKit.WebPolicyDecision decision){
		string uri = req.get_uri();
		if(uri != static_uri){
			Diorite.System.open_uri_with_fallback(uri);
			decision.ignore();
			return true;
		}
		return false;
	}
}
