namespace Nuvola{

private const string APP_CRASHED = "crashed";
private const string APP_CHECK_CRASHED = "check_crashed";
private const string LAST_SERVICE_KEY = "last_service";

namespace Actions{
	public const string PURGE_COOKIES = "purge-cookies";
	public const string PURGE_CACHE = "purge-cache";
	public const string PURGE_SETTINGS = "purge-settings";
	public const string PURGE_UI_STATE = "purge-ui-state";
	public const string DISABLE_PROXY = "disable-proxy";
}

/**
 * Struct containing command line arguments. Check Args.options for their meaning.
 */
struct Args {
	static bool debug = true;
	static bool verbose = false;
	static bool version = false;
	static bool time = false;
	static bool autoplay = false;
	static bool noautoplay = false;
	static bool debug_javascript = true;
	static bool debug_navigation = true;
	static bool debug_http = true;
	static bool no_web_plugins = false;
}

private const string TMP_DIR = "build/tests/tmp";
const string DATA_USER = "dirs/data_user";
const string DATA_DIR_1 = "dirs/data_dir_1";
const string DATA_DIR_2 = "dirs/data_dir_2";
const string CONFIG_USER = "dirs/config_user";
const string CACHE_USER = "dirs/cache_user";

// Top directory
string TOP;

int main(string[] args){
	/* Set top directory */
	TOP = Environment.get_current_dir();
	if(Path.get_basename(TOP) == "build"){
		TOP = Path.get_dirname(TOP);
	}
	
	/* Set XDG directories */
	Environment.set_variable("XDG_DATA_HOME", @"$TOP/$TMP_DIR/$DATA_USER", true);
	Environment.set_variable("XDG_CONFIG_HOME", @"$TOP/$TMP_DIR/$CONFIG_USER", true);
	Environment.set_variable("XDG_CACHE_HOME", @"$TOP/$TMP_DIR/$CACHE_USER", true);
	Environment.set_variable("XDG_DATA_DIRS", @"$TOP/$TMP_DIR/$DATA_DIR_1:$TOP/$TMP_DIR/$DATA_DIR_2", true);
	
	purge_test_dir();
	
	Test.init(ref args);
	TestSuite root = TestSuite.get_root();
	
	root.add_suite(new Nuvola.ServiceTest().get_suite());
	root.add_suite(new Nuvola.ServicesManagerTest().get_suite());
	root.add_suite(new Nuvola.Extensions.Lyrics.CachedLyricsTest().get_suite());
	root.add_suite(new Nuvola.Extensions.Lyrics.AZLyricsTest().get_suite());
	
	return Test.run();
}


public File create_test_dir(string? subdir=null){
	var dir = File.new_for_path(TOP + "/" + TMP_DIR);
	if(subdir != null){
		dir = dir.get_child(subdir);
	}
	
	if(!dir.query_exists()){
		try{
			dir.make_directory_with_parents();
		}
		catch(Error e){
			error("Unable to create directory %s: %s", dir.get_path(), e.message);
		}
	}
	return dir;
}

public void purge_test_dir(){
	var dir = File.new_for_path(TOP + "/" + TMP_DIR);
	Diorite.System.try_purge_dir(dir);
}

} // namespace Nuvola
