using FsoFramework.Test;

public class Nuvola.ServiceTest: FsoFramework.Test.TestCase{
	private const string SERVICE_ID = "test_service";

	public ServiceTest(){
		base("ServiceTest");
		add_test("new_service", test_new_service);
	}
	
	public override void set_up(){
	}
	
	public override void tear_down(){
		purge_test_dir();
	}
	
	private void assert_invalid_service(Diorite.MultiTypeMap metadata, File service_dir, File conf_file, string reason){
		try{
			new Service(SERVICE_ID, metadata, conf_file, service_dir, false);
			Test.message("Service should be invalid, because %s, but no error thrown.", reason);
			Test.fail();
		}
		catch(ServiceError e){
			Test.message(@"Caught error: #$(e.code): %s", e.message);
		}
	}
	
	public void test_new_service(){
		var service_dir = create_test_dir("service").get_child(SERVICE_ID);
		var metadata_file = service_dir.get_child("metadata.conf");
		var metadata = new Diorite.MapFile(metadata_file, false);
		var conf_file = create_test_dir("config").get_child(@"$(SERVICE_ID).json");
		
		/* Loading of service from non-existent directory must fail. */
		assert_invalid_service(metadata, service_dir, conf_file, "doesn't exist");
		
		/* Copy test service. */
		string spawn_stdout = null;
		string spawn_stderr = null;
		int result = 0;
		try{
			string[] args = {"cp", "-rf", TOP + "/tests/data/services/" + SERVICE_ID, service_dir.get_path()};
			Process.spawn_sync(null, args, null,  SpawnFlags.SEARCH_PATH, null, out spawn_stdout, out spawn_stderr, out result);
		}
		catch(SpawnError e){
			Test.message("Failed to copy service: %s", spawn_stderr ?? "null");
			Test.fail();
		}
		
		/* Reload metadata. */
		metadata = new Diorite.MapFile(metadata_file, false);
		
		/* Valid service must not fail. */
		try{
			new Service(SERVICE_ID, metadata, conf_file, service_dir, false);
		}
		catch(ServiceError e){
			Test.message(@"Loading of valid service failed: #$(e.code): %s", e.message);
			Test.fail();
		}
		
		try{
			
			/* Test check of mandatory fields */
			string[] mandatory_fields = {"name", "version", "home_page", "sandbox_pattern", "maintainer_name", "maintainer_link"};
			foreach(string k in mandatory_fields){
				string val = metadata[k];
				metadata[k] = "";
				metadata.save();
				assert_invalid_service(metadata, service_dir, conf_file, k + " field missing");
				metadata[k] = val ?? "";
			}
			
			/* Test validity check */
			var key = "sandbox_pattern";
			var value = metadata[key];
			metadata[key] = "foo***";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
			
			key = "version";
			value = metadata[key];
			metadata[key] = "0";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
			
			key = "version_minor";
			value = metadata[key];
			metadata[key] = "-1";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
			
			key = "api_major";
			value = metadata[key];
			metadata[key] = "0";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
			
			key = "api_minor";
			value = metadata[key];
			metadata[key] = "-1";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
			
			key = "api_major";
			value = metadata[key];
			metadata[key] = "10";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
			
			key = "api_minor";
			value = metadata[key];
			metadata[key] = "999";
			metadata.save();
			assert_invalid_service(metadata, service_dir, conf_file, key + " field is invalid");
			metadata[key] = value ?? "";
		}
		catch(Error e){
			Test.message("Unexpected error: %s", e.message);
			Test.fail();
		}
	}
}
