#!/bin/bash
# Creates service package for Nuvola Player
# Copyright 2012 Jiří Janoušek <janousek.jiri@gmail.com>
# License: http://www.opensource.org/licenses/bsd-license.php

# debug
#~ set -x

if [[ -z "$1" ]]; then
  echo "
    Usage: $0 SERVICE_ID

This script build service integration package that contains

  * directory with service integration files for service SERVICE_ID
  * installation script - service-package/install_service.sh
  * installation instructions - service-package/README.txt

The package is intended to be used to

  * release fixes of service integration without need to
    create a bug fix release of Nuvola Player
  * backport new service integrations to older releases
"
  exit 1
fi

# wroking directory
WD=$PWD

# service id
SID="$1"

cd "$WD/data/nuvolaplayer/services"

# service version
SVERSION=`grep version "$SID/metadata.conf" | cut -d '=' -f 2 | tr -d ' '`

# tar archive name
ARCHIVE="$WD/nuvolaplayer-$SID-$SVERSION.tar"

# add service to the archive
tar cvf $ARCHIVE $SID

cd $WD/tools/service-package

# add installation script to the archive
tar rvf $ARCHIVE *

# compress archive
gzip -f $ARCHIVE
